//
// Created by lukas on 09.03.21.
//

#ifndef LEDSTRIPINTERFACE_INFORMATIONSERVICE_H
#define LEDSTRIPINTERFACE_INFORMATIONSERVICE_H

#include <HttpEndpoint.h>
#include <WebSocketTxRx.h>
#include "Clock.h"

#define DEFAULT_CLOCK_STATE true
#define OFF_STATE "OFF"
#define ON_STATE "ON"

#define INFORMATION_SETTINGS_SOCKET_PATH "/ws/information"

class InformationState {
 public:
  bool wordClockOn;
  bool twentyAfterSyntax;
  bool uhrAlwaysOn;

  static void read(InformationState& settings, JsonObject& root) {
    root["wordclockOn"] = settings.wordClockOn;
    root["twentyAfterSyntax"] = settings.twentyAfterSyntax;
    root["uhrAlwaysOn"] = settings.uhrAlwaysOn;
  }

  static StateUpdateResult update(JsonObject& root, InformationState& lightState) {
    const bool newOnState = root["wordclockOn"] | DEFAULT_CLOCK_STATE;
    const bool newTwentyAfterState = root["twentyAfterSyntax"] | DEFAULT_CLOCK_STATE;
    const bool newUhrAlwaysOnState = root["uhrAlwaysOn"] | DEFAULT_CLOCK_STATE;

    if (lightState.wordClockOn != newOnState || lightState.twentyAfterSyntax != newTwentyAfterState || lightState.uhrAlwaysOn != newUhrAlwaysOnState) {
      lightState.wordClockOn = newOnState;
      lightState.twentyAfterSyntax = newTwentyAfterState;
      lightState.uhrAlwaysOn = newUhrAlwaysOnState;
      return StateUpdateResult::CHANGED;
    }
    return StateUpdateResult::UNCHANGED;
  }

//  static void haRead(InformationState& settings, JsonObject& root) {
//    root["state"] = settings.wordClockOn ? ON_STATE : OFF_STATE;
//  }
//
//  static StateUpdateResult haUpdate(JsonObject& root, InformationState& lightState) {
//    String state = root["state"];
//    // parse new led state
//    boolean newState = false;
//    if (state.equals(ON_STATE)) {
//      newState = true;
//    } else if (!state.equals(OFF_STATE)) {
//      return StateUpdateResult::ERROR;
//    }
//    // change the new state, if required
//    if (lightState.wordClockOn != newState) {
//      lightState.wordClockOn = newState;
//      return StateUpdateResult::CHANGED;
//    }
//    return StateUpdateResult::UNCHANGED;
//  }
};

class InformationService : public StatefulService<InformationState>{
 public:
  InformationService(AsyncWebServer* server, SecurityManager* securityManager);
  void begin();

 private:
  WebSocketTxRx<InformationState> _webSocket;

  Clock clock;

  void onConfigUpdated();
};

#endif  // LEDSTRIPINTERFACE_INFORMATIONSERVICE_H
